<?php
/**
* Plugin Name: Encuentra tu tienda
* Description: Un buscador de tiendas usando Google Maps
* Version: 1.0
* Author: The Back Room
*/

class encuentraTienda {

  private $today;
  private $todayplus3;
  private $dates;
  private $menu_hook;

  public function __construct() {

    // STYLES & SCRIPTS
    add_action( 'wp_enqueue_scripts', array($this, 'stylesScripts' ) );
    add_action( 'admin_enqueue_scripts', array($this, 'stylesScriptsAdmin' ) );

    // SHORTCODE
    add_shortcode( "encuentra-tienda", array($this, 'codeHTML' ) );
    add_shortcode( "proximas-aperturas", array($this, 'codeNewShops' ) );
    add_shortcode( "horarios-especiales", array($this, 'codeSpecial' ) );

    // AJAX Handlers
		add_action( 'wp_ajax_resultSpecial', array( $this, 'resultSpecial' ));
		add_action( 'wp_ajax_nopriv_resultSpecial', array( $this, 'resultSpecial' ));

    // AJAX Handlers backend
		add_action( 'wp_ajax_json_tiendas_pregenerator', array( $this, 'json_tiendas_pregenerator' ));
		add_action( 'wp_ajax_nopriv_json_tiendas_pregenerator', array( $this, 'json_tiendas_pregenerator' ));

		// Post Type Creation
		add_action( 'init', array($this, 'create_post_type' ) );

    // Search filter
    add_filter( 'posts_where', array( $this, 'rel_search_where' ) );
    add_filter( 'posts_join', array( $this, 'rel_search_join' ) );
    add_filter( 'posts_groupby', array( $this, 'rel_search_dupe_fix' ) );

    // Adding code colum to post list
    add_filter('manage_posts_columns', array( $this, 'code_columns_head' ) );
    add_action('manage_posts_custom_column', array( $this, 'code_columns_content'), 10, 2);

    // REST API
    add_action( 'rest_api_init', array( $this, 'endpointCode' ) );
  }


	// Registering Styles & Scripts
  public function stylesScripts() {

    // STYLES
    wp_register_style( 'encuentra-tienda', plugin_dir_url( __FILE__ ) . "css/encuentra-tienda.css", array(), time() );
    wp_register_style( 'aperturas', plugin_dir_url( __FILE__ ) . "css/aperturas.css", array(), time() );
    wp_register_style( 'especiales', plugin_dir_url( __FILE__ ) . "css/especiales.css", array(), time() );
		wp_register_style( 'font-awesome-animation', plugin_dir_url( __FILE__ ) . "css/font-awesome-animation.min.css", array(), time() );

    // SCRIPTS
		wp_register_script( 'markclusterer', plugin_dir_url( __FILE__ ) . 'js/markclusterer_compiled.js', array('jquery'), '', true );
		// wp_register_script( 'google-maps', 'https://maps.google.com/maps/api/js?v=3.22&sensor=false&libraries=geometry,places&language=es&key=AIzaSyCGk-wvkrRflib37SXKGMJ3fo4j0UHNAZI', array('markclusterer'), '', true);
		wp_register_script( 'json-style-maps', plugin_dir_url( __FILE__ ) . 'json/style_map.json', array('jquery','markclusterer'), '', true );
    wp_register_script( 'encuentra-tienda', plugin_dir_url( __FILE__ ) . 'js/encuentra-tienda.js', array('jquery','markclusterer','json-style-maps'/*,'json-tiendas'*/), filemtime(plugin_dir_path( __FILE__ ) . 'js/encuentra-tienda.js'), true );
    wp_register_script( 'aperturas', plugin_dir_url( __FILE__ ) . 'js/aperturas.js', array('jquery','markclusterer','json-style-maps'), filemtime(plugin_dir_path( __FILE__ ) . 'js/aperturas.js'), true );
    wp_register_script( 'especiales', plugin_dir_url( __FILE__ ) . 'js/especiales.js', array('jquery','markclusterer','json-style-maps'), '', true );

		wp_localize_script( 'encuentra-tienda', 'E_Tienda', array(

			// Route to the images
      'ruta' => plugin_dir_url( __FILE__ ) . "images/",

    ));

    wp_localize_script( 'aperturas', 'E_Tienda', array(

			// Route to the images
      'ruta' => plugin_dir_url( __FILE__ ) . "images/",
      'home' => get_home_url(),
      'ap_img' => get_field('ap_img')['url'],
      'ap_txt1' => get_field('ap_txt1')

    ));

    wp_localize_script( 'especiales', 'H_Especiales', array(
      // URL to wp-admin/admin-ajax.php to process the request

      'ajaxurl' => admin_url( 'admin-ajax.php' ),

      // generate a nonce with a unique ID "myajax-post-comment-nonce"
      // so that you can check it later when an AJAX request is sent
      'security' => wp_create_nonce( 'special-ajax' )

    ));

  }

  public function stylesScriptsAdmin() {

    wp_enqueue_style( 'backend', plugin_dir_url( __FILE__ ) . "css/backend.css" );
    wp_enqueue_script( 'backend', plugin_dir_url( __FILE__ ) . 'js/backend.js' );
    wp_localize_script( 'backend', 'G_Json', array(
      // URL to wp-admin/admin-ajax.php to process the request

      'ajaxurl' => admin_url( 'admin-ajax.php' ),

      // generate a nonce with a unique ID "myajax-post-comment-nonce"
      // so that you can check it later when an AJAX request is sent
      'security' => wp_create_nonce( 'gjson-ajax' )

    ));

  }

	// It creates a post type called 'Tiendas'
	public function create_post_type() {

	  register_post_type( 'tienda',
	    array(
	      'labels' => array(
	        'name' => __( 'Tiendas' ),
	        'singular_name' => __( 'Tienda' )
	      ),
	      'public' => true,
	      'has_archive' => true,
        'menu_icon'   => 'dashicons-store',
        'show_in_rest' => true,
        'rest_base' => 'tiendas'
	    )
	  );

	}

	public function codeHTML() {

		// GENERATE SHOPS
		add_action('wp_footer', array($this, 'generate_shops') );

		// Enqueing styles & scripts
    wp_enqueue_script( 'json-tiendas' );
		wp_enqueue_style( 'encuentra-tienda' );
		wp_enqueue_style( 'font-awesome-animation' );
		wp_enqueue_script( 'markclusterer' );
		wp_enqueue_script( 'google-maps' );
		wp_enqueue_script( 'json-style-maps' );
		wp_enqueue_script( 'encuentra-tienda' );
	  ?>

    <div id="go-top"></div>
		<div class="encuentratutienda">

			<div id="flex">
				<div id="search">
					<div class="box-search">
						<form name="miform" method="post" onsubmit="return false;">
							<input type="hidden" name="frominput" id="frominput">
							<input type="text" name="location" id="location" class="location" placeholder="Encuentra tu tienda ..." onkeypress="return doSearch(event);"><a href="javascript:void(0)"  onclick="geocode('location')" class="btn_search"><i class="fa fa-search"></i></a>
						</form>
						<a href="javascript:void(0)" class="geocode" onclick="miUbicacion()"><span id="buscando">Buscar por tu ubicación</span> <i id="miubicacion">&nbsp;</i></a>
					</div>
					<div id="modo">
						<ul><li id="modo-mapa" class="active"><a href="javascript:void(0)" onclick="showMapa()" class="selected"><i class="fa fa-map-marker"></i>&nbsp;&nbsp;MAPA<span></span></a></li><li id="modo-listado" class="active"><a href="javascript:void(0)" onclick="showListado()" class="noselected"><i class="fa fa-list"></i>&nbsp;&nbsp;LISTADO<span></span></a></li></ul>
					</div>
				</div>
				<div id="mapa"></div>
			</div>

			<div id="listado">
				<ul id="list">
				</ul>
        <!-- LISTA DE MÁS DE 10 DÍAS -->

				<div id="info">
					<div class="info-cap">
						<a href="javascript:void(0)" onclick="hideDetail()" class="close mypulse"><span class="hidden">[CLOSE]</span></a>
						<h2><span class="hidden">ALDI</span></h2>
						<h3><!--MASQUEFA--></h3>
					</div>
					<ul class="info-tab">
						<li id="modo-info-1"><a href="javascript:void(0)" onclick="showInfo(1)" class="selected">INFORMACIÓN</a></li><li id="modo-info-2"><a href="javascript:void(0)" onclick="showInfo(2)">CÓMO LLEGAR</a></li>
					</ul>
          <div class="info-0" id="info-1">

          </div>
					<div class="info-0" id="info-2">
						<h4>DIRECCIÓN</h4>
						<form name="miformComoLlegar" id="miformComoLlegar" method="post">
							<input type="hidden" name="travelmodeinput" id="travelmodeinput">
							<div><label for="location_a">A</label><input type="text" name="location_a" id="location_a" class="location" placeholder="Salida" onkeypress="return doSearch2(event);" onfocus="focusLocation()"><a href="javascript:void(0)" class="geocode_a" onclick="miUbicacion2('')" id="miubicacion2">.</a></div>
							<div><label for="location_b">B</label><input type="text" name="location_b" id="location_b" class="location" placeholder="Destino" value="Tienda seleccionada" disabled></div>
						</form>
						<ul class="travelmode">
							<li><a href="javascript:void(0)" onclick="getIndicaciones('driving')" id="driving" class="faa-parent animated-hover"><i class="fa fa-car faa-pulse"></i><span class="hidden">[CAR]</span></a></li><li><a href="javascript:void(0)" onclick="getIndicaciones('walking')" id="walking" class="faa-parent animated-hover"><i class="fa fa-male faa-pulse"></i><span class="hidden">[WALK]</span></a></li><li><a href="javascript:void(0)" onclick="getIndicaciones('transit')" id="transit" class="faa-parent animated-hover"><i class="fa fa-subway faa-pulse"></i><span class="hidden">[TRANSIT]</span></a></li>
						</ul>
						<div id="dir-container"></div>
						<div id="map-container"></div>
					</div>
				</div>
			</div>

		</div>
	<?php
	}

  // HTML Code for the new shops
  public function codeNewShops() {

    // GENERATE SHOPS
		add_action('wp_footer', array($this, 'generate_new_shops') );

    // Enqueing styles & scripts
		wp_enqueue_style( 'aperturas' );
		wp_enqueue_style( 'font-awesome-animation' );
		wp_enqueue_script( 'markclusterer' );
		wp_enqueue_script( 'google-maps' );
		wp_enqueue_script( 'json-style-maps' );
		wp_enqueue_script( 'aperturas' );
    ?>

      <div class="container">
          <div id="aperturas">

              <div id="modo">
                  <ul><li id="modo-listado" class="active"><a href="javascript:void(0)" onclick="showListado()" class="selected"><i class="fa fa-list"></i>&nbsp;&nbsp;LISTADO<span></span></a></li><li id="modo-mapa" class="active"><a href="javascript:void(0)" onclick="showMapa()" class="noselected"><i class="fa fa-map-marker"></i>&nbsp;&nbsp;MAPA<span></span></a></li></ul>
              </div>

              <div class="aperturas-info">

                  <div id="list"></div>

              </div>

              <!-- Aquí se muestran los mapas -->
              <div class="maps-wrapper">
                <div class="buble-aperturas">

                  <div class="info-0" id="como-llegar">
                      <div id="aperturas-info">
                      </div>

                      <h4>DIRECCIÓN</h4>
                      <form name="miform" id="miformComoLlegar" method="post">
                        <input type="hidden" name="frominput" id="frominput">
                          <input type="hidden" name="travelmodeinput" id="travelmodeinput">
                          <div><label for="location_a">A</label><input type="text" name="frominput" id="location_a" class="location" placeholder="Salida" onkeypress="return doSearch2(event);"
                                                                       onfocus="focusLocation()"><a href="javascript:void(0)" class="geocode_a" onclick="miUbicacion2('')"
                                                                                                    id="miubicacion2">.</a></div>
                          <div><label for="location_b">B</label><input type="text" name="location_b" id="location_b" class="location" placeholder="Destino" value="Tienda seleccionada"
                                                                       disabled></div>
                      </form>
                      <ul class="travelmode">
                          <li><a href="javascript:void(0)" onclick="getIndicaciones('driving')" id="driving" class="faa-parent animated-hover"><i class="fa fa-car faa-pulse"></i><span
                                          class="hidden">[CAR]</span></a></li>
                          <li><a href="javascript:void(0)" onclick="getIndicaciones('walking')" id="walking" class="faa-parent animated-hover"><i class="fa fa-male faa-pulse"></i><span
                                          class="hidden">[WALK]</span></a></li>
                          <li><a href="javascript:void(0)" onclick="getIndicaciones('transit')" id="transit" class="faa-parent animated-hover"><i class="fa fa-subway faa-pulse"></i><span
                                          class="hidden">[TRANSIT]</span></a></li>
                      </ul>
                      <div id="dir-container"></div>
                      <div id="map-container"></div>
                  </div>
                </div>

                <div id="mapa"></div>
              </div>

              <div id="list10"></div>

          </div>

            <div class="aperturas-vacio">
                <div class="aperturas-vacio-inner">
                    <h2>¡Vaya!</h2>
                    <p>Ahora mismo no hay ninguna apertura a la vista pero estamos trabajando para estar cada día más cerca de ti. Mientras tanto, puedes encontrar tu tienda más cercana en <a href="<?php echo get_permalink(1339)?>" class="how-to-arrive">este buscador</a>.</p>
                </div>
            </div>
      </div>

  <?php
  }

	// It generates a shops array in json format taking the shops from the Wordpress Custom Type
	public function generate_shops() {

		$args = array (
			'post_type' => 'tienda',
			'posts_per_page' => -1,
      'post_status' => 'publish'
		);

		$query = new WP_Query( $args );

		echo '<script>';
		echo 'var dataMap = { "tiendas": [';

		if( $query->have_posts() )
		{
			while ( $query->have_posts() ) { $query->the_post();

				$shopID = get_the_ID();
				?>    {
					 "codigo":"<?php echo addslashes(get_field('ti_cod',$shopID)); ?>"
					,"nombre":"<?php echo addslashes(get_the_title()); ?>"
					,"direccion":"<?php echo addslashes(strip_tags(trim(get_field('ti_dir',$shopID)))); ?>"
					,"latitude":<?php echo get_field('ti_lat',$shopID)."\n"; ?>
					,"longitude":<?php echo get_field('ti_lon',$shopID)."\n"; ?>
					,"link":"<?php echo addslashes(get_field('ti_link',$shopID)); ?>"
					,"localidad":"<?php echo addslashes(get_field('ti_loc',$shopID)); ?>"
					,"provincia":"<?php echo addslashes(get_field('ti_pro',$shopID)); ?>"
					,"codpostal":"<?php echo addslashes(get_field('ti_cp',$shopID)); ?>"
					,"telefono":"<?php echo addslashes(get_field('ti_tel',$shopID)); ?>"
					,"horario":"<?php echo addslashes(get_field('ti_hor',$shopID)); ?>"
					,"festivos":"<?php echo addslashes(get_field('ti_fes',$shopID)); ?>"
					,"nueva":"<?php echo addslashes(get_field('ti_nue',$shopID)); ?>"
				},
			<?php
			}
		}

		wp_reset_postdata();


		echo ']};';
		echo '</script>';

	}

  // HTML Code the special time table
  public function codeSpecial() {

    // Enqueing styles & scripts
		wp_enqueue_style( 'especiales' );
		wp_enqueue_script( 'especiales' );
    ?>

    <div class="he-filters">
      <?php
      //***** FILTRO PROVINCIAS *****
      $args = array (
  			'post_type' => 'tienda',
  			'posts_per_page' => -1,
      );
      $the_query = new WP_Query( $args );

      if( $the_query->have_posts() ) {
        $provincias = array();

        while( $the_query->have_posts() ) { $the_query->the_post(); // Adding to the provinces the terms, and ignoring the existing ones
          $provincias[] = get_field('ti_pro',$the_query->ID);
        }
        $provincias = array_unique($provincias);
        $collator = collator_create("es");
        $collator->sort($provincias);
        ?>
        <span class="select-decorator">
          <select name="provincia" id="especial-provincia">
            <option value="vacio"><?php _e("Escoge provincia"); ?></option>
            <?php foreach ($provincias as $provincia) { ?>
              <option value="<?php echo $provincia; ?>"><?php echo __("Provincia de ").$provincia; ?></option>
            <?php } ?>
          </select>
        </span>
        <?php
      }

      $festivos = $this->specialDates();
      $festivos = $this->sortDates($festivos);
      $nombresFestivos = $this->formatDate($festivos);

      if( !empty($nombresFestivos) ) {
        ?>
        <span class="select-decorator">
          <select name="festivo" id="especial-festivo">
            <option value="vacio">Escoge festivo</option>
            <?php foreach ($nombresFestivos as $key => $festivo) { ?>
              <?php $current_month = explode('/', $festivos[$key] )[1]; ?>
              <?php if( $current_month == '01' || $current_month == '12' ) { ?>
                <option value="<?php echo $festivos[$key]; ?>"><?php echo $nombresFestivos[$key]; ?></option>
              <?php } ?>
            <?php } ?>
          </select>
        </span>
        <?php
      }
      wp_reset_postdata();
      ?>
    </div>
    <?php
    echo "<div class='special-results'>";
    $this->printSpecials();
    echo "</div>";

  }

  // Getting special dates
  public function specialDates() {

    $dates = array();
    $onlydates = array();

    $args = array (
      'post_type' => 'tienda',
      'posts_per_page' => -1,
      'orderby' => 'date',
      'order' => 'ASC'
    );
    $the_query = new WP_Query( $args );

    if( $the_query->have_posts() ) {

      while( $the_query->have_posts() ) { $the_query->the_post(); // Adding to the provinces the terms, and ignoring the existing ones

        $string = addslashes(get_field('ti_fes',$the_query->ID));

        $dates[] = $this->separateComma($string);

      }
      $onlydates = $this->getOnlyDates($dates);
    }

    return $onlydates;

  }

  // Separating by comma
  public function separateComma($string) {

    if($string != "") {

      // Dividing by comma
      $tdates = substr_count($string,',');

      $from = 0;
      for($i=0;$i<=$tdates;$i++) {

        if($i == $tdates) {

          $substr = substr($string,$from,(strlen($string)-$commapos));

        } else {

          $commapos = strpos($string,",",$from);
          $substr = substr($string,$from,($commapos-$from));
          $from = $commapos+1;

        }
        $dates[] = $substr;

      }
      $dates[] = "25/12";
      $dates[] = "01/01";
      $dates[] = "06/01";

    }

    return $dates;

  }

  // It gets the special date from a matrix, skipping the hours
  public function getOnlyDates($dates) {

    foreach($dates as $date) {

      if( !empty( $date ) ) {

        foreach($date as $d) {

          $posdates = strpos($d,':');

          if($posdates!=false) {
            $onlydates[] = substr($d,0,$posdates);
          } else {
            $onlydates[] = $d;
          }
          $onlydates = array_slice(array_unique(array_filter($onlydates)),0);

        }

      }

    }

    return $onlydates;

  }

  // It gets the special date from an array, skipping the hours
  public function getOnlyDate($dates) {

    foreach($dates as $date) {

      $posdates = strpos($date,':');

      if($posdates) {
        $onlydates[] = substr($date,0,$posdates);
      } else {
        $onlydates[] = $date;
      }

      $onlydates = array_slice(array_unique(array_filter($onlydates)),0);

    }

    return $onlydates;

  }

  // Sorting the array of dates with the format "dd/mm"
  public function sortDates($dates) {

    foreach($dates as $date) {

      $day = substr($date,0,2);
      $month = substr($date,3,2);

      //$dateObj[]   = DateTime::createFromFormat('m-d', $month."-".$day);
      if($month==1) {
        $dateObj[] = "2017".$month.$day;
      } else {
        $dateObj[] = "2016".$month.$day;
      }

      $dateOriginal[] = $date;

    }

    $i = 1;
    $n = count($dateObj);

    while( ($i<$n) && ($ordenado==false) ) {

      $i++;
      $ordenado = true;

      for($j=0;$j<($n-1);$j++) {
        if($dateObj[$j]>$dateObj[$j+1]) {
          $ordenado = false;

          // Sorting the array
          $aux = $dateObj[$j];
          $dateObj[$j] = $dateObj[$j+1];
          $dateObj[$j+1] = $aux;

          // Sorting the original array
          $aux = $dateOriginal[$j];
          $dateOriginal[$j] = $dateOriginal[$j+1];
          $dateOriginal[$j+1] = $aux;
        }
      }

    }

    return $dateOriginal;

  }

  // It transform the date format dd/mm into text
  public function formatDate($dates) {

    foreach($dates as $date) {

      $day = substr($date,0,2);
      $month = substr($date,3,2);
      $monthName = $this->monthName($month);

      $formatted[] = $day." de ".$monthName;

    }
    return $formatted;

  }

  // It gets the string of the dates and gives back the hours in proper format
  public function getHours($date) {

    if(!empty($date)) {

      $startingpoint = strpos($date,':');

      if($startingpoint!=0) {

        $open = substr($date,$startingpoint+1,5);
        $close = substr($date,$startingpoint+7,5);
        return $open." - ".$close;

      } else {

        return "<div class='special-closed'>Cerrado</div>";

      }

    } else {
      return "";
    }

  }

  // Getting the shop with special time table
  public function resultSpecial() {

    check_ajax_referer( 'special-ajax', 'security' );
    $this->printSpecials($_POST['provincia'],$_POST['festivo']);
    die(); // Wordpress requeriment for using AJAX

  }

  // It prints the special results
  public function printSpecials($provincia=null,$festivo=null) {

    if( $provincia==null || $festivo==null || $provincia=="vacio" || $festivo=="vacio" ) { // If the user needs to select options

      echo "<div class='fill-everything'>Por favor, escoge una provincia y un festivo para recibir los datos.</div>";

    } elseif( $festivo=="25/12" || $festivo=="01/01" || $festivo=="06/01" ) {

      echo "<div class='no-aperturable'>Festivo, cerrado.</div>";

    } else {

      // Getting the results
      $args = array (
        'post_type' => 'tienda',
        'posts_per_page' => -1,
        'orderby' => 'title',
        'order' => 'ASC'
      );
      $the_query = new WP_Query( $args );

      if( $the_query->have_posts() ) {

        $somethingfound = false;

        $resultTable = "<table class='table-special'>";
        $resultTable .= "<thead>";
        $resultTable .= "<tr>";
        $resultTable .= "<td>TIENDA</td><td>HORARIOS</td>";
        $resultTable .= "</tr>";
        $resultTable .= "</thead>";
        $resultTable .= "<tbody>";
        while( $the_query->have_posts() ) { $the_query->the_post(); // Adding to the provinces the terms, and ignoring the existing ones

          $datefound = false;
          $provfound = false;

          $date = addslashes(get_field('ti_fes',$the_query->ID));
          $prov = get_field('ti_pro',$the_query->ID);

          $onlydates = $this->separateComma($date);
          $onlydates2 = $this->getOnlyDate($onlydates);

          foreach($onlydates2 as $key => $d) {
            if($d==$festivo) {
              $datefound = true;
              $currenthours = $onlydates[$key];
              break;
            }
          }

          if($prov == $provincia) {
            $provfound = true;
          }

          // Results found, we print the results in a table
          if($datefound && $provfound) {
            $somethingfound = true;
            $resultTable .= "<tr>";

            $resultTable .= "<td><h3>".get_the_title()."</h3><div class='address'>".strip_tags(get_field('ti_dir',$the_query->ID))."<br>".get_field('ti_cp',$the_query->ID)." ".get_field('ti_loc',$the_query->ID)."</div></td>";
            $resultTable .= "<td>".$this->getHours($currenthours)."</td>";

            $resultTable .= "</tr>";
          }

        }
        $resultTable .= "</tbody>";
        $resultTable .= "</table>";

      }

      // Checking if there was found any result
      if( $somethingfound == false ) {
          echo "<div class='no-special-results'>En esta fecha nuestras tiendas abren en su horario habitual, puedes consultarlo <a href='https://www.aldi.es/encuentra-tu-tienda/'>aquí</a>.</div>";
      } else {
        echo $resultTable;
      }

    }

  }

  public function currentDate() {

    $todayN = getdate();
    if($todayN['mday']<10) {
      $day = "0".$todayN['mday'];
    } else {
      $day = $todayN['mday'];
    }
    $this->today = $todayN['year'].$todayN['mon'].$day;

    //Calculating 3 days less
    $day3 = $day-3;
    if( $day3 <= 0 ) {

      $month3 = $todayN['mon'] - 1;

      if($month3 <= 0) {

        $month3 = 12;

        $year3 = $todayN['year'] - 1;

        $day3 = cal_days_in_month( CAL_GREGORIAN, $month3, $todayN['year'] );

        if($day3<10) {
          $day3="0".$day3;
        }
        if($month3<10) {
          $month3="0".$month3;
        }

        $this->todayless3 = $year3.$month3.$day3;

      } else {


        $day3 = cal_days_in_month( CAL_GREGORIAN, $month3, $todayN['year']);
        if($day3<10) {
          $day3="0".$day3;
        }
        if($month3<10) {
          $month3="0".$month3;
        }
        $this->todayless3 = $todayN['year'].$month3.($day3);

      }


    } else {

      if($todayN['mon']<10) {
        $monthC="0".$todayN['mon'];
      } else {
        $monthC=$todayN['mon'];
      }
      if($day3<10) {
        $day3="0".$day3;
      }
      $this->todayless3 = $todayN['year'].$monthC.$day3;

    }
    echo "<script>console.log('".$this->todayless3." ".$this->today."');</script>";

  }

  // It generates a shops array in json format taking the shops from the Wordpress Custom Type
	public function generate_new_shops() {

      $this->currentDate();
      echo "<script>console.log('".$this->todayless3."')</script>";
  		$args = array (
  			'post_type' => 'tienda',
  			'posts_per_page' => -1,
        'meta_query' => array (

          array(
            'key' => 'ti_nue',
            'value' => $this->todayless3,
            'compare' => '>='
          )

        ),
        'meta_key' => 'ti_nue',
        'orderby' => 'meta_value',
        'order' => 'ASC',
        'post_status' => 'publish'

  		);

  		$query = new WP_Query( $args );

  		echo '<script>';
  		echo 'var dataMap = { "tiendas": [';

  		if( $query->have_posts() )
  		{
  			while ( $query->have_posts() ) { $query->the_post();
  				$shopID = get_the_ID();
  				?>
  				{
  					 "codigo":"<?php echo addslashes(get_field('ti_cod',$shopID)); ?>"
  					,"nombre":"<?php echo addslashes(get_the_title()); ?>"
  					,"direccion":"<?php echo addslashes(strip_tags(trim(get_field('ti_dir',$shopID)))); ?>"
  					,"latitude":<?php echo get_field('ti_lat',$shopID)."\n"; ?>
  					,"longitude":<?php echo get_field('ti_lon',$shopID)."\n"; ?>
  					,"link":"<?php echo addslashes(get_field('ti_link',$shopID)); ?>"
  					,"localidad":"<?php echo addslashes(get_field('ti_loc',$shopID)); ?>"
  					,"provincia":"<?php echo addslashes(get_field('ti_pro',$shopID)); ?>"
  					,"codpostal":"<?php echo addslashes(get_field('ti_cp',$shopID)); ?>"
  					,"telefono":"<?php echo addslashes(get_field('ti_tel',$shopID)); ?>"
  					,"horario":"<?php echo addslashes(get_field('ti_hor',$shopID)); ?>"
  					,"festivos":"<?php echo addslashes(get_field('ti_fes',$shopID)); ?>"
  					,"nueva":"<?php echo addslashes(get_field('ti_nue',$shopID)); ?>"
            ,"revista":"<?php echo (get_field('ti_rev',$shopID)) ? get_post_permalink(get_field('ti_rev',$shopID)->ID) : '' ; ?>"
  				},
  			<?php
  			}
  		}

  		wp_reset_postdata();


  		echo ']};';
  		echo '</script>';

  	}

    public function monthName($number) {
      $meses = array("enero","febrero","marzo","abril","mayo","junio","julio","agosto","septiembre","octubre","noviembre","diciembre");

      return $meses[$number-1];
    }

    /**************************** BACKEND TASKS *******************************/

    /********* SEARCH BY ANOTHER COLUMNS **********/
    public function is_tienda_search() {

      global $pagenow;
      $its = false;

      if ( is_admin() && $pagenow == 'edit.php' && $_GET['post_type'] == 'tienda' && $_GET['s'] != '') {
        $its = true;
      }

      return $its;

    }

    public function rel_search_join( $join ) {

      global $wpdb;

      if ( $this->is_tienda_search() ) {
          $join .= 'LEFT JOIN ' . $wpdb->postmeta . ' ON '. $wpdb->posts . '.ID = ' . $wpdb->postmeta . '.post_id ';
      }

      return $join;

    }

    public function rel_search_where( $where ) {

      global $wpdb;

      if ( $this->is_tienda_search() ) {
          $where = preg_replace( "/\(\s*".$wpdb->posts.".post_title\s+LIKE\s*(\'[^\']+\')\s*\)/", "(".$wpdb->posts.".post_title LIKE $1) OR (".$wpdb->postmeta.".meta_value LIKE $1)", $where );
          $where = str_replace( "OR wp_posts.post_status = 'pending'", "", $where );
          $where = str_replace( "OR wp_posts.post_status = 'private'", "", $where );
          $where = str_replace( "OR wp_posts.post_status = 'draft'", "", $where );
          $where = str_replace( "OR wp_posts.post_status = 'future'", "", $where );
      }

      return $where;

    }

    public function rel_search_dupe_fix( $groupby ) {

      global $wpdb;

      if ( $this->is_tienda_search() ) {
        $groupby = "$wpdb->posts.ID";
      }

      return $groupby;
    }

    /************ ADDING CODE COLUMN TO LIST ************/
    // ADD NEW COLUMN
    public function code_columns_head($defaults) {

      if( isset($_GET['post_type']) && ( $_GET['post_type'] == "tienda" ) ) {
        $defaults['code_tienda'] = 'Código';
      }

      return $defaults;

    }

    // SHOW THE FEATURED IMAGE
    public function code_columns_content($column_name, $post_ID) {

      if( isset($_GET['post_type']) && ( $_GET['post_type'] == "tienda" ) ) {
        if ($column_name == 'code_tienda') {
            $code_tienda = get_post_meta($post_ID,'ti_cod',true);
            if ($code_tienda) {
                echo $code_tienda;
            }
        }
      }

    }

    /******************************* JSON FILE ********************************/
    // Adding the menu page to the "Tiendas" menu
    public function ett_add_admin_menu(  ) {

    	$this->menu_hook = add_submenu_page( 'edit.php?post_type=tienda', 'Generar JSON', 'Generar JSON', 'edit_pages', 'generar_json_tiendas', array( $this, 'ett_options_page' ) );
      add_action( 'load-'.$this->menu_hook, array( $this,'delete_cron_on_update_settings' ) );

    }

    // Deleting the cron settings in case that the user changed the settings
    public function delete_cron_on_update_settings()
    {

      if(isset($_GET['settings-updated']) && $_GET['settings-updated']) {
        wp_clear_scheduled_hook( 'json_daily_creation' );

        $this->json_tiendas_scheduler();
      }

    }

    public function ett_settings_init(  ) {

    	register_setting( 'ettPage', 'ett_settings' );

    	add_settings_section(
    		'ett_ettPage_section',
    		__( 'Periodicidad', 'ett' ),
    		array( $this, 'ett_settings_section_callback' ),
    		'ettPage'
    	);

    	add_settings_field(
    		'ett_text_field_0',
    		__( 'Periodicidad', 'ett' ),
    		array( $this, 'ett_text_field_0_render' ),
    		'ettPage',
    		'ett_ettPage_section'
    	);

    }

    public function ett_text_field_0_render(  ) {

    	$options = get_option( 'ett_settings' );
    	?>
      <select name='ett_settings[ett_select_time]'>
        <option value='oneminute' <?php selected( $options['ett_select_time'], "oneminute" ); ?>>Cada minuto</option>
        <option value='onehour' <?php selected( $options['ett_select_time'], "onehour" ); ?>>Cada hora</option>
    		<option value='daily' <?php selected( $options['ett_select_time'], "daily" ); ?>>Diariamente</option>
    		<option value='weekly' <?php selected( $options['ett_select_time'], "weekly" ); ?>>Semanalmente</option>
        <option value='monthly' <?php selected( $options['ett_select_time'], "monthly" ); ?>>Mensualmente</option>
    	</select>

    	<?php

    }

    public function ett_settings_section_callback(  ) {

    	echo __( 'Establece la periodicidad con la que se ejecutará automaticamente la creación del fichero JSON con los datos actualizados de las tiendas.', 'ett' );

    }

    public function ett_options_page(  ) {

    	?>
    	<form action='options.php' method='post'>

    		<h2>Generar JSON de tiendas</h2>

    		<?php
    		settings_fields( 'ettPage' );
    		do_settings_sections( 'ettPage' );
    		submit_button();
    		?>

    	</form>

      <div class="next-time-json">
        <?php
        if(wp_next_scheduled('json_daily_creation')) { // Getting next time the cron will run
          echo "El fichero se generará automáticamente a <strong>".date("d/m/Y - G:i",wp_next_scheduled('json_daily_creation'))."</strong>";
        } else {
          echo "<div class='alert'>No hay una fecha esperada para la generación automática del fichero JSON de tiendas.<br><strong>Si acabas de cambiar la periodicidad, por favor actualiza esta página para ver la próxima ejecución</strong></div>";
        }
        ?>
      </div>

      <div class="generar-tiendas-json">
        <a class="generar-tiendas-json-ahora">Generar ahora</a><div class="generar-tiendas-json-msg"></div><br>
        <a class="ver-json" href="<?php echo plugin_dir_url( __FILE__ ).'data_map.json' ?>" target="_blank">Ver el fichero JSON</a>
      </div>

    	<?php

    }

    public function json_tiendas_scheduler() {

      $options = get_option( 'ett_settings' );

      if( $options['ett_select_time'] ) {
        $time = $options['ett_select_time']; // Getting the time range
      } else {
        $time = "daily";
      }

      if ( ! wp_next_scheduled( 'json_daily_creation' ) ) {
        wp_schedule_event( time(), $time, 'json_daily_creation' );
      }

    }

    // It connects with the ajax answer
    public function json_tiendas_pregenerator() {

      check_ajax_referer( 'gjson-ajax', 'security' );
      $this->json_tiendas_generator();
      die();

    }

    // It generates the file
    public function json_tiendas_generator() {

      $args = array (
  			'post_type' => 'tienda',
  			'posts_per_page' => -1
  		);

  		$query = new WP_Query( $args );

      $datamap = 'var dataMap = { "tiendas": [';

  		if( $query->have_posts() )
  		{

  			while ( $query->have_posts() ) { $query->the_post();

  				$shopID = get_the_ID();
  				$datamap .= '{';
  				$datamap .= '"codigo":"'.addslashes(get_field('ti_cod',$shopID)).'"';
  				$datamap .= ', "nombre":"'.addslashes(get_the_title()).'"';
  				$datamap .= ', "direccion":"'.addslashes(strip_tags(trim(get_field('ti_dir',$shopID)))).'"';
  				$datamap .= ', "latitude":'.get_field('ti_lat',$shopID);
  				$datamap .= ', "longitude":'.get_field('ti_lon',$shopID);
  				$datamap .= ', "link":"'.addslashes(get_field('ti_link',$shopID)).'"';
  				$datamap .= ', "localidad":"'.addslashes(get_field('ti_loc',$shopID)).'"';
  				$datamap .= ', "provincia":"'.addslashes(get_field('ti_pro',$shopID)).'"';
  				$datamap .= ', "codpostal":"'.addslashes(get_field('ti_cp',$shopID)).'"';
  				$datamap .= ', "telefono":"'.addslashes(get_field('ti_tel',$shopID)).'"';
  				$datamap .= ', "horario":"'.addslashes(get_field('ti_hor',$shopID)).'"';
  				$datamap .= ', "festivos":"'.addslashes(get_field('ti_fes',$shopID)).'"';
  				$datamap .= ', "nueva":"'.addslashes(get_field('ti_nue',$shopID)).'"';
  				$datamap .= '}, ';

  			}

  		}

      $datamap .= ']};';

  		wp_reset_postdata();

      $fp = fopen(PLUGIN_DIR.'/data_map.json', 'w');

      if(fwrite($fp, $datamap)) {
        echo "Se ha generado el archivo satisfactoriamente.";
      } else {
        echo "No se ha podido actualizar el archivo. Por favor, consulta con el administrador del sistema.";
      }

      fclose($fp);

    }

    function my_add_intervals( $schedules ) {
    	// add a 'weekly' interval
    	$schedules['weekly'] = array(
    		'interval' => 604800,
    		'display' => __('Once Weekly')
    	);
    	$schedules['monthly'] = array(
    		'interval' => 2635200,
    		'display' => __('Once a month')
    	);
      $schedules['oneminute'] = array(
    		'interval' => 60,
    		'display' => __('Once every minute')
    	);
      $schedules['onehour'] = array(
    		'interval' => 360,
    		'display' => __('Once every hour')
    	);
    	return $schedules;
    }

    // REST API ENDPOINTS
    public function endpointCode() {

      register_rest_route( 'wp/v2/', 'tiendas/code/(?P<id>.+)', array(
        'methods' => 'GET',
        'callback' => array( $this, 'getStoreId'),

        )
      );

    }

    public function getStoreId( WP_REST_Request $request ) {

      $args = array(
        'post_type'		=>	'tienda',
        'meta_query'	=>	array(
          array(
            'key' => 'ti_cod',
            'value'	=>	$request['id'],
            'compare' => '='
          )
        ),
        'post_status' => array('publish', 'pending', 'draft', 'auto-draft', 'future', 'private', 'inherit', 'trash')
      );

      $my_query = new WP_Query( $args );

      if( $my_query->have_posts() ) {
        while( $my_query->have_posts() ) {
          $my_query->the_post();
          echo get_the_ID()." ";
        } // end while
      } // end if
      wp_reset_postdata();

      die();

    }

    /* Filter the single-template with our custom function */
    public function custom_survey( $single ) {

        global $wp_query, $post;

        /* Checks for single template by post type */
        // Recipes
        if ($post->post_type == "receta") {

            if(file_exists( plugin_dir_path( __FILE__ ) . 'single-receta.php' )) {
              return plugin_dir_path( __FILE__ ) . 'single-receta.php';
            }

        }

        // DIY
        if ( $post->post_type == "diy" ) {

            if(file_exists( plugin_dir_path( __FILE__ ) . 'single-diy.php' )) {
              return plugin_dir_path( __FILE__ ) . 'single-diy.php';
            }

        }

        // Posts
        if ( $post->post_type == "post" ) {

            if(file_exists( plugin_dir_path( __FILE__ ) . 'single-post.php' )) {
              return plugin_dir_path( __FILE__ ) . 'single-post.php';
            }

        }
        return $single;

    }

}

define( 'PLUGIN_DIR', dirname(__FILE__) . DIRECTORY_SEPARATOR);
define( 'ETT_PLUGIN_URL', plugin_dir_url(__FILE__) );

require_once( PLUGIN_DIR . '/acf-fields.php');
require_once( PLUGIN_DIR . '/openings-survey.php' );

$ett = new encuentraTienda();
